@@ -11,24 +11,7 @@ INCLUDES = """
 TYPES = """
 typedef ... BN_CTX;
 typedef ... BIGNUM;
-/*
- * TODO: This typedef is wrong.
- *
- * This is due to limitations of cffi.
- * See https://bitbucket.org/cffi/cffi/issue/69
- *
- * For another possible work-around (not used here because it involves more
- * complicated use of the cffi API which falls outside the general pattern used
- * by this package), see
- * http://paste.pound-python.org/show/iJcTUMkKeBeS6yXpZWUU/
- *
- * The work-around used here is to just be sure to declare a type that is at
- * least as large as the real type.  Maciej explains:
- *
- * <fijal> I think you want to declare your value too large (e.g. long)
- * <fijal> that way you'll never pass garbage
- */
-typedef uintptr_t BN_ULONG;
+typedef int... BN_ULONG;
 """
 
 FUNCTIONS = """
@@ -88,6 +71,8 @@ int BN_mask_bits(BIGNUM *, int);
 """
 
 MACROS = """
+int BN_num_bytes(const BIGNUM *);
+
 int BN_zero(BIGNUM *);
 int BN_one(BIGNUM *);
 int BN_mod(BIGNUM *, const BIGNUM *, const BIGNUM *, BN_CTX *);
