@@ -12,17 +12,19 @@ from .utils import check_backend_support, select_backends, skip_if_empty
 
 
 def pytest_generate_tests(metafunc):
-    names = metafunc.config.getoption("--backend")
-    selected_backends = select_backends(names, _available_backends())
-
     if "backend" in metafunc.fixturenames:
+        names = metafunc.config.getoption("--backend")
+        selected_backends = select_backends(names, _available_backends())
+
         filtered_backends = []
         required = metafunc.function.requires_backend_interface
-        required_interfaces = tuple(
+        required_interfaces = [
             mark.kwargs["interface"] for mark in required
-        )
+        ]
         for backend in selected_backends:
-            if isinstance(backend, required_interfaces):
+            if all(
+                isinstance(backend, iface) for iface in required_interfaces
+            ):
                 filtered_backends.append(backend)
 
         # If you pass an empty list to parametrize Bad Things(tm) happen
