@@ -8,8 +8,6 @@ import pretend
 
 import pytest
 
-import six
-
 from cryptography import utils
 from cryptography.exceptions import AlreadyFinalized, _Reasons
 from cryptography.hazmat.backends.interfaces import HashBackend
@@ -32,7 +30,7 @@ class TestHashContext(object):
     def test_hash_reject_unicode(self, backend):
         m = hashes.Hash(hashes.SHA1(), backend=backend)
         with pytest.raises(TypeError):
-            m.update(six.u("\u00FC"))
+            m.update(u"\u00FC")
 
     def test_copy_backend_object(self):
         backend = DummyHashBackend([hashes.SHA1])
